/* * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. The Blender
 * Foundation also sells licenses for use in proprietary software under
 * the Blender License.  See http://www.blender.org/BL/ for information
 * about this.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
 * All rights reserved.
 *
 * The Original Code is: all of this file.
 *
 * Contributor(s): Joseph Gilbert
 *
 * ***** END GPL/BL DUAL LICENSE BLOCK *****
 *
 */

#ifndef EXPP_euler_h
#define EXPP_euler_h

#include "Python.h"
#include "BKE_utildefines.h"
#include "gen_utils.h"
#include "Types.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*****************************/
//    Euler Python Object   
/*****************************/

#define EulerObject_Check(v) ((v)->ob_type == &euler_Type)

typedef struct {
	PyObject_VAR_HEAD
	float * vec;
} EulerObject;

//doc strings
static char Euler_Zero_doc[] =
"() - set all values in the euler to 0";

static char Euler_Canonical_doc[] =
"() - sets the euler rotation a unique shortest arc rotation - tests for gimbal lock";

static char Euler_ToMatrix_doc[] =
"() - returns a rotation matrix representing the euler rotation";

static char Euler_ToQuat_doc[] =
"() - returns a quaternion representing the euler rotation";

//prototypes
PyObject *newEulerObject(float *vec);
static PyObject *Euler_Zero(PyObject *self, PyObject *args);
static PyObject *Euler_Canonical(PyObject *self, PyObject *args);
static PyObject *Euler_ToMatrix(PyObject *self, PyObject *args);
static PyObject *Euler_ToQuat(PyObject *self, PyObject *args);

#endif /* EXPP_euler_h */

