/* $Id: vector.h,v 1.4 2003/07/10 20:00:51 michel Exp $
 *
 * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. The Blender
 * Foundation also sells licenses for use in proprietary software under
 * the Blender License.  See http://www.blender.org/BL/ for information
 * about this.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
 * All rights reserved.
 *
 * The Original Code is: all of this file.
 *
 * Contributor(s): Joseph Gilbert
 *
 * ***** END GPL/BL DUAL LICENSE BLOCK *****
 *
 */

#ifndef EXPP_matrix_h
#define EXPP_matrix_h

#include "Python.h"

/*****************************/
/*    Matrix Python Object   */
/*****************************/
typedef float **ptRow;

typedef struct _Matrix {
	PyObject_VAR_HEAD

	ptRow matrix;
	int rowSize;
	int colSize;
	int flag;
		//0 - no coercion
		//1 - coerced from int
		//2 - coerced from float
} MatrixObject;


//doc strings
static char Matrix_Zero_doc[] =
"() - set all values in the matrix to 0";

static char Matrix_Identity_doc[] =
"() - set the square matrix to it's identity matrix";

static char Matrix_Transpose_doc[] =
"() - set the matrix to it's transpose";

static char Matrix_Determinant_doc[] =
"() - return the determinant of the matrix";

static char Matrix_Inverse_doc[] =
"() - set the matrix to it's inverse if an inverse is possible";

static char Matrix_TranslationPart_doc[] = 
"() - return a vector encompassing the translation of the matrix";

static char Matrix_RotationPart_doc[] = 
"() - return a vector encompassing the rotation of the matrix";

static char Matrix_Resize4x4_doc[] = 
"() - resize the matrix to a 4x4 square matrix";

static char Matrix_toEuler_doc[] = 
"() - convert matrix to a euler angle rotation";

static char Matrix_toQuat_doc[] = 
"() - convert matrix to a quaternion rotation";
/*****************************************************************************/
/* Python API function prototypes.												*/
/*****************************************************************************/
PyObject *newMatrixObject(float * mat, int size);
void init_py_matrix(void);
int Matrix_CheckPyObject (PyObject *pyobj);
static PyObject *Matrix_Zero(MatrixObject *self, PyObject *args);
static PyObject *Matrix_Identity(MatrixObject *self, PyObject *args);
static PyObject *Matrix_Transpose(MatrixObject *self, PyObject *args);
static PyObject *Matrix_Determinant(MatrixObject *self, PyObject *args);
static PyObject *Matrix_Inverse(MatrixObject *self, PyObject *args);
static PyObject *Matrix_TranslationPart(MatrixObject *self, PyObject *args);
static PyObject *Matrix_RotationPart(MatrixObject *self, PyObject *args);
static PyObject *Matrix_Resize4x4(MatrixObject *self, PyObject *args);
static PyObject *Matrix_toEuler(MatrixObject *self, PyObject *args);
static PyObject *Matrix_toQuat(MatrixObject *self, PyObject *args);

#endif /* EXPP_matrix_H */