/* 
 *
 * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. The Blender
 * Foundation also sells licenses for use in proprietary software under
 * the Blender License.  See http://www.blender.org/BL/ for information
 * about this.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
 * All rights reserved.
 *
 * The Original Code is: all of this file.
 *
 * Contributor(s): Joseph Gilbert
 *
 * ***** END GPL/BL DUAL LICENSE BLOCK *****
 *
 */

#ifndef EXPP_quat_h
#define EXPP_quat_h

#include "Python.h"
#include "BKE_utildefines.h"
#include "gen_utils.h"
#include "Types.h"
#include "BLI_winstuff.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*****************************/
//    Quaternion Python Object   
/*****************************/

#define QuaternionObject_Check(v) ((v)->ob_type == &quaternion_Type)

typedef struct {
	PyObject_VAR_HEAD
	float * vec;
	int flag;
		//0 - no coercion
		//1 - coerced from int
		//2 - coerced from float
} QuaternionObject;

//doc strings
static char Quaternion_Identity_doc[] =
"() - set the quaternion to it's identity (1, vector)";

static char Quaternion_Negate_doc[] =
"() - set all values in the quaternion to their negative";

static char Quaternion_Conjugate_doc[] =
"() - set the quaternion to it's conjugate";

static char Quaternion_Inverse_doc[] =
"() - set the quaternion to it's inverse";

static char Quaternion_Normalize_doc[] =
"() - normalize the vector portion of the quaternion";

static char Quaternion_ToEuler_doc[] =
"() - return a euler rotation representing the quaternion";

static char Quaternion_ToMatrix_doc[] =
"() - return a rotation matrix representing the quaternion";

//prototypes
PyObject *newQuaternionObject(float *vec);
static PyObject *Quaternion_Identity(PyObject *self, PyObject *args);
static PyObject *Quaternion_Negate(PyObject *self, PyObject *args);
static PyObject *Quaternion_Conjugate(PyObject *self, PyObject *args);
static PyObject *Quaternion_Inverse(PyObject *self, PyObject *args);
static PyObject *Quaternion_Normalize(PyObject *self, PyObject *args);
static PyObject *Quaternion_ToEuler(PyObject *self, PyObject *args);
static PyObject *Quaternion_ToMatrix(PyObject *self, PyObject *args);

#endif /* EXPP_quat_h */

